/* protocols.h */
#ifndef _NETINET_PROTOCOLS_H
#define _NETINET_PROTOCOLS_H

#define IP_ICMP		  1
#define IP_IGMP		  2
#define IP_GGP 		  3
#define IP_ST  		  5
#define IP_TCP 		  6
#define IP_UCL 		  7
#define IP_EGP 		  8
#define IP_IGP 		  9
#define IP_BBN_RCC_MON	 10
#define IP_NVP_II	 11
#define IP_PUP		 12
#define IP_ARGUS	 13
#define IP_EMCON	 14
#define IP_XNET		 15
#define IP_CHAOS	 16
#define IP_UDP		 17
#define IP_MUX		 18
#define IP_DCN_MEAS	 19
#define IP_HMP		 20
#define IP_PRM		 21
#define IP_XNS_IDP	 22
#define IP_TRUNK1	 23
#define IP_TRUNK2	 24
#define IP_LEAF1	 25
#define IP_LEAF2	 26
#define IP_RDP		 27
#define IP_IRTP		 28
#define IP_ISO_TP4	 29
#define IP_NETBLT	 30
#define IP_MFE_NSP	 31
#define IP_MERIT_INP	 32
#define IP_SEP		 33
#define IP_3PC		 34
#define IP_CFTP		 62
#define SAT_EXPAK	 64
#define IP_RVD		 66
#define IP_IPPC		 67
#define IP_SAT_MON	 69
#define IP_VISA		 70
#define IP_IPCV		 71
#define IP_BR_SAT_MON	 76
#define IP_SUN_ND	 77
#define IP_WB_MON	 78
#define IP_WB_EXPAK	 79
#define IP_ISO_IP	 80
#define IP_VMTP		 81
#define IP_SECURE_VMTP	 82
#define IP_VINES	 83
#define IP_TTP		 84
#define NSFNET_IGP	 85
#define IP_DGP		 86
#define IP_TCF		 87
#define IP_IGRP		 88
#define IP_OSPFIGP	 89
#define IP_SPRITE_RPG	 90
#define IP_LARP		 91

#endif /*  _NETINET_PROTOCOLS_H*/
