/* Copyright (C) 1991, 1994 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <ansidecl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>

/* Return the username of the caller.
   If S is not NULL, it points to a buffer of at least L_cuserid bytes
   into which the name is copied; otherwise, a static buffer is used.  */
char *cuserid( char * );

char *
DEFUN(cuserid, (s), char *s)
{
  static char name[L_cuserid];
  char user_name [L_cuserid];
  char *login_name = getlogin ();
  uid_t user_uid;
  /* We need that for POSIX. */
  struct passwd *pwent = getpwuid(geteuid());
  
  /* We should not do anything. */
  if (pwent == NULL)
  {
    if (s != NULL)
      s[0] = '\0';
    return NULL;
  }

  /* We get effective user name and ID. */
  strcpy (user_name, pwent->pw_name);
  user_uid = pwent->pw_uid;

  /* It is tricky here. We don't want to use getlogin () if
   * user_uid != getpwnam(login_name)->pw_uid.
   */
  if (login_name == NULL || (pwent = getpwnam(login_name)) == NULL
	|| user_uid != pwent->pw_uid)
  {
    login_name = user_name;
  }

  if (s == NULL)
    s = name;

  return strcpy(s, login_name);
}
