$set 7 #AuthList

$ #1 Original Message:(Authentication OK)
# Authentifizierung erfolgreich

$ #2 Original Message:(Invalid client credential)
# Client Bestätigung ungültig

$ #3 Original Message:(Server rejected credential)
# Server: Bestätigung zurückgewiesen

$ #4 Original Message:(Invalid client verifier)
# Client Überprüfung ungültig

$ #5 Original Message:(Server rejected verifier)
# Server: Überprüfung zurückgewiesen

$ #6 Original Message:(Client credential too weak)
# Client Bestätigung zu unsicher

$ #7 Original Message:(Invalid server verifier)
# Server Überprüfung ungültig

$ #8 Original Message:(Failed (unspecified error))
# Fehlgeschlagen (nicht spezifizierter Fehler)

