#ifndef _PTHREAD_MIT_SYS_TYPES_H
#define _PTHREAD_MIT_SYS_TYPES_H

typedef unsigned int pthread_size_t;
typedef int pthread_ssize_t;
typedef unsigned int pthread_ipaddr_type;
typedef unsigned short  pthread_ipport_type;
typedef long pthread_clock_t;
typedef long pthread_time_t;
typedef long pthread_fpos_t;
typedef long pthread_off_t;
typedef void * pthread_va_list;

#endif /* _PTHREAD_MIT_SYS_TYPES_H */
