$set 1 #ErrorList

$ #0 Original Message:(Unknown error)
# Erreur inconnue

$ #1 Original Message:(Operation not permitted)
# Opération non autorisée

$ #2 Original Message:(No such file or directory)
# Fichier ou répertoire inéxistant

$ #3 Original Message:(No such process)
# Processus inexistant

$ #4 Original Message:(Interrupted system call)
# Appel système interrompu

$ #5 Original Message:(I/O error)
# Erreur E/S

$ #6 Original Message:(No such device or address)
# Périphérique ou adresse inexistant

$ #7 Original Message:(Arg list too long)
# Liste d'arguments trop longue

$ #8 Original Message:(Exec format error)
# Erreur sur le format exécutable

$ #9 Original Message:(Bad file number)
# Mauvais numéro de fichier

$ #10 Original Message:(No child processes)
# Pas de processus fils

$ #11 Original Message:(Try again)
# Essayez encore

$ #12 Original Message:(Out of memory)
# Plus de mémoire

$ #13 Original Message:(Permission denied)
# Permission refusée

$ #14 Original Message:(Bad address)
# Mauvaise adresse

$ #15 Original Message:(Block device required)
# Périphérique bloc requis

$ #16 Original Message:(Device or resource busy)
# Périphérique ou ressource occupé

$ #17 Original Message:(File exists)
# Le fichier existe

$ #18 Original Message:(Cross-device link)
# Lien entre plusieurs périphériques

$ #19 Original Message:(No such device)
# Périphérique inexistant

$ #20 Original Message:(Not a directory)
# N'est pas un répertoire

$ #21 Original Message:(Is a directory)
# Est un répertoire

$ #22 Original Message:(Invalid argument)
# Argument invalide

$ #23 Original Message:(File table overflow)
# Dépassement de la table des fichiers

$ #24 Original Message:(Too many open files)
# Trop de fichiers ouverts

$ #25 Original Message:(Not a typewriter)
# N'est pas un terminal

$ #26 Original Message:(Text file busy)
# Fichier code occupé

$ #27 Original Message:(File too large)
# Fichier trop grand

$ #28 Original Message:(No space left on device)
# Plus d'espace libre sur le périphérique

$ #29 Original Message:(Illegal seek)
# seek illégal

$ #30 Original Message:(Read-only file system)
# Système de fichier en lecture seulement

$ #31 Original Message:(Too many links)
# Trop de liens

$ #32 Original Message:(Broken pipe)
# Rupture du tuyau

$ #33 Original Message:(Math argument out of domain of func)
# Argument mathématique en dehors du domaine de définition de la fonction

$ #34 Original Message:(Math result not representable)
# Résultat mathématique non représentable

$ #35 Original Message:(Resource deadlock would occur)
# Blocage mutuel de la ressource

$ #36 Original Message:(File name too long)
# Nom du fichier trop long

$ #37 Original Message:(No record locks available)
# Pas de verrou disponible

$ #38 Original Message:(Function not implemented)
# Fonction non implémenté

$ #39 Original Message:(Directory not empty)
# Répertoire non vide

$ #40 Original Message:(Too many symbolic links encountered)
# Trop de liens symboliques rencontrés

$ #41 Original Message:(Operation would block)
# L'opération devrait bloquer

$ #42 Original Message:(No message of desired type)
# Pas de messages du type désiré

$ #43 Original Message:(Identifier removed)
# Identificateur supprimé

$ #44 Original Message:(Channel number out of range)
# Numéro du canal en dehors des limites

$ #45 Original Message:(Level 2 not synchronized)
# Niveau 2 non synchronisé

$ #46 Original Message:(Level 3 halted)
# Niveau 3 arrêté

$ #47 Original Message:(Level 3 reset)
# Niveau 3 remis à zéro

$ #48 Original Message:(Link number out of range)
# Numéro du lien en dehors des limites

$ #49 Original Message:(Protocol driver not attached)
# Contrôleur de protocole non attaché

$ #50 Original Message:(No CSI structure available)
# Pas de structure CSI disponible

$ #51 Original Message:(Level 2 halted)
# Niveau 2 arrêté

$ #52 Original Message:(Invalid exchange)
# Echange invalide

$ #53 Original Message:(Invalid request descriptor)
# Demande de descripteur invalide

$ #54 Original Message:(Exchange full)
# Echange plein

$ #55 Original Message:(No anode)
# Pas d'anode

$ #56 Original Message:(Invalid request code)
# Code de demande invalide

$ #57 Original Message:(Invalid slot)
# Emplacement invalide

$ #58 Original Message:(File locking deadlock error)
# Erreur sur un interblocage de vérou

$ #59 Original Message:(Bad font file format)
# Mauvais format de fichier fonte

$ #60 Original Message:(Device not a stream)
# Le périphérique n'est pas un stream

$ #61 Original Message:(No data available)
# Pas de données disponibles

$ #62 Original Message:(Timer expired)
# Durée expirée

$ #63 Original Message:(Out of streams resources)
# Plus de ressources stream

$ #64 Original Message:(Machine is not on the network)
# La machine n'est pas sur le réseau

$ #65 Original Message:(Package not installed)
# Package non installé

$ #66 Original Message:(Object is remote)
# Objet à distance

$ #67 Original Message:(Link has been severed)
# Le lien a été coupé

$ #68 Original Message:(Advertise error)
# Erreur d'annonce

$ #69 Original Message:(Srmount error)
# Erreur srmount

$ #70 Original Message:(Communication error on send)
# Erreur de communication lors d'un envoi

$ #71 Original Message:(Protocol error)
# Erreur de protocole

$ #72 Original Message:(Multihop attempted)
# Tentative de sauts multiples

$ #73 Original Message:(RFS specific error)
# Erreur spécifique à RFS

$ #74 Original Message:(Not a data message)
# N'est pas un message de donnée

$ #75 Original Message:(Value too large for defined data type)
# Valeur trop grande pour le type de données définies

$ #76 Original Message:(Name not unique on network)
# Nom non unique sur le réseau

$ #77 Original Message:(File descriptor in bad state)
# Descripteur de fichier dans un mauvais état

$ #78 Original Message:(Remote address changed)
# Adresse à distance changée

$ #79 Original Message:(Can not access a needed shared library)
# Ne peut accéder à une librairie partagée nécessaire

$ #80 Original Message:(Accessing a corrupted shared library)
# Accès à une librairie altérée

$ #81 Original Message:(.lib section in a.out corrupted)
# La section .lib dans un a.out est altérée

$ #82 Original Message:(Attempting to link in too many shared libraries)
# Tentative de lier trop de librairies partagées

$ #83 Original Message:(Cannot exec a shared library directly)
# Impossibilté d'exécuter directement une librairie partagée

$ #84 Original Message:(Illegal byte sequence)
# Séquence d'octet illégale

$ #85 Original Message:(Interrupted system call should be restarted)
# L'appel système interrompu ne peut être repris

$ #86 Original Message:(Streams pipe error)
# Erreur d'un tuyau de stream

$ #87 Original Message:(Too many users)
# Trop d'utilisateurs

$ #88 Original Message:(Socket operation on non-socket)
# Opération de type socket sur un fichier non-socket

$ #89 Original Message:(Destination address required)
# Adresse de destination obligatoire

$ #90 Original Message:(Message too long)
# Message trop long

$ #91 Original Message:(Protocol wrong type for socket)
# Mauvais type de protocole pour un socket

$ #92 Original Message:(Protocol not available)
# Protocole non disponible

$ #93 Original Message:(Protocol not supported)
# Protocole non supporté

$ #94 Original Message:(Socket type not supported)
# Type de socket non supporté

$ #95 Original Message:(Operation not supported on transport endpoint)
# Opération non supportée sur l'extrémité du transport

$ #96 Original Message:(Protocol family not supported)
# Famille de protocoles non supportée

$ #97 Original Message:(Address family not supported by protocol)
# Famille d'adresses non supportée par le protocole

$ #98 Original Message:(Address already in use)
# Adresse déjà utilisée

$ #99 Original Message:(Cannot assign requested address)
# Ne peut pas assigner l'adresse demandée

$ #100 Original Message:(Network is down)
# Le réseau est éteint

$ #101 Original Message:(Network is unreachable)
# Le réseau ne peut être atteint

$ #102 Original Message:(Network dropped connection because of reset)
# Le réseau a rompu la connection à cause d'une remise à zéro

$ #103 Original Message:(Software caused connection abort)
# Echec de connection à cause du logiciel

$ #104 Original Message:(Connection reset by peer)
# Connection remise à zéro par le correspondant

$ #105 Original Message:(No buffer space available)
# Pas d'espace disponible dans le buffer

$ #106 Original Message:(Transport endpoint is already connected)
# L'extrémité du transport est déjà connectée

$ #107 Original Message:(Transport endpoint is not connected)
# L'extrémité du transport n'est pas connectée

$ #108 Original Message:(Cannot send after transport endpoint shutdown)
# Impossibilité d'envoyer après un arrêt de l'extrémité du transport

$ #109 Original Message:(Too many references: cannot splice)
# Trop de références: cannot splice

$ #110 Original Message:(Connection timed out)
# Durée écoulée pour la connection

$ #111 Original Message:(Connection refused)
# Connection refusée

$ #112 Original Message:(Host is down)
# L'hôte est éteint

$ #113 Original Message:(No route to host)
# Aucune route pour cet hôte

$ #114 Original Message:(Operation already in progress)
# Operation déjà en cours

$ #115 Original Message:(Operation now in progress)
# Operation maintenant en cours

$ #116 Original Message:(Stale NFS file handle)
# Handle de fichier NFS perdu

$ #117 Original Message:(Structure needs cleaning)
# Structure a besoin de nettoyage

$ #118 Original Message:(Not a XENIX named type file)
# N'est pas un fichier XENIX de type nommé

$ #119 Original Message:(No XENIX semaphores available)
# Aucun sémaphore XENIX disponible

$ #120 Original Message:(Is a named type file)
# Est un fichier de type nommé

$ #121 Original Message:(Remote I/O error)
# Erreur sur E/S eloignée
