$set 1 #ErrorList

$ #0 Original Message:(Unknown error)
# Неизвестная ошибка

$ #1 Original Message:(Operation not permitted)
# Операция не разрешена

$ #2 Original Message:(No such file or directory)
# Hет такого файла или каталога

$ #3 Original Message:(No such process)
# Hет такого процесса

$ #4 Original Message:(Interrupted system call)
# Прерванный системный вызов

$ #5 Original Message:(I/O error)
# Ошибка ввода/вывода

$ #6 Original Message:(No such device or address)
# Hет такого устройства или адреса

$ #7 Original Message:(Arg list too long)
# Слишком длинный список аргументов

$ #8 Original Message:(Exec format error)
# Ошибочный формат исполняемого файла

$ #9 Original Message:(Bad file number)
# Недопустимый номер файла

$ #10 Original Message:(No child processes)
# Нет дочерних процессов

$ #11 Original Message:(Try again)
# Повторите ещё раз

$ #12 Original Message:(Out of memory)
# Не хватает памяти

$ #13 Original Message:(Permission denied)
# Доступ запрещен

$ #14 Original Message:(Bad address)
# Недопустимый адрес

$ #15 Original Message:(Block device required)
# Требуется блочное устройство

$ #16 Original Message:(Device or resource busy)
# Устройство или ресурс заняты

$ #17 Original Message:(File exists)
# Такой файл уже есть

$ #18 Original Message:(Cross-device link)
# Недопустимая ссылка на другое устройство

$ #19 Original Message:(No such device)
# Hет такого устройства

$ #20 Original Message:(Not a directory)
# Это не каталог

$ #21 Original Message:(Is a directory)
# Это каталог

$ #22 Original Message:(Invalid argument)
# Hеправильный аргумент

$ #23 Original Message:(File table overflow)
# Переполнена таблица файлов

$ #24 Original Message:(Too many open files)
# Слишком много открытых файлов

$ #25 Original Message:(Not a typewriter)
# Это не пишущая машинка

$ #26 Original Message:(Text file busy)
# Файл текста занят

$ #27 Original Message:(File too large)
# Файл слишком велик

$ #28 Original Message:(No space left on device)
# Hа устройстве не осталось места

$ #29 Original Message:(Illegal seek)
# Недопустимое позиционирование

$ #30 Original Message:(Read-only file system)
# Файловая система только для чтения

$ #31 Original Message:(Too many links)
# Слишком много ссылок

$ #32 Original Message:(Broken pipe)
# Оборванный конвейер

$ #33 Original Message:(Math argument out of domain of func)
# Математический аргумент функции вне области определения

$ #34 Original Message:(Math result not representable)
# Математический результат непредставим

$ #35 Original Message:(Resource deadlock would occur)
# Произойдет взаимная блокировка ресурсов

$ #36 Original Message:(File name too long)
# Слишком длинное имя файла

$ #37 Original Message:(No record locks available)
# Исчерпаны блокировки записей

$ #38 Original Message:(Function not implemented)
# Эта функция не реализована

$ #39 Original Message:(Directory not empty)
# Этот каталог непуст

$ #40 Original Message:(Too many symbolic links encountered)
# Встретилось слишком много символических ссылок

$ #41 Original Message:(Operation would block)
# Эта операция может быть заблокирована

$ #42 Original Message:(No message of desired type)
# Нет сообщений требуемого типа

$ #43 Original Message:(Identifier removed)
# Этот идентификатор удалён

$ #44 Original Message:(Channel number out of range)
# Недопустимый номер канала

$ #45 Original Message:(Level 2 not synchronized)
# Уровень 2 не синхронизирован

$ #46 Original Message:(Level 3 halted)
# Уровень 3 остановлен

$ #47 Original Message:(Level 3 reset)
# Уровень 3 переустановлен

$ #48 Original Message:(Link number out of range)
# Недопустимый номер ссылки

$ #49 Original Message:(Protocol driver not attached)
# Драйвер протокола не присоединён

$ #50 Original Message:(No CSI structure available)
# Структура CSI недоступна

$ #51 Original Message:(Level 2 halted)
# Уровень 2 остановлен

$ #52 Original Message:(Invalid exchange)
# Недопустимая перестановка

$ #53 Original Message:(Invalid request descriptor)
# Недопустимый дескриптор запроса

$ #54 Original Message:(Exchange full)
# Перестановка переполнена

$ #55 Original Message:(No anode)
# Нет анода

$ #56 Original Message:(Invalid request code)
# Недопустимый код запроса

$ #57 Original Message:(Invalid slot)
# Недопустимый слот

$ #58 Original Message:(File locking deadlock error)
# Взаимоблокировка файла

$ #59 Original Message:(Bad font file format)
# Недопустимый формат файла шрифтов

$ #60 Original Message:(Device not a stream)
# Это устройство не является потоком

$ #61 Original Message:(No data available)
# Нет данных

$ #62 Original Message:(Timer expired)
# Время таймера истекло

$ #63 Original Message:(Out of streams resources)
# Исчерпаны ресурсы птоков

$ #64 Original Message:(Machine is not on the network)
# Эта машина не подключена к сети

$ #65 Original Message:(Package not installed)
# Этот пакет не установлен

$ #66 Original Message:(Object is remote)
# Этот объект является удалённым

$ #67 Original Message:(Link has been severed)
# Эта ссылка была ужесточена

$ #68 Original Message:(Advertise error)
# Ошибка объявления

$ #69 Original Message:(Srmount error)
# Srmount error

$ #70 Original Message:(Communication error on send)
# Коммуникационная ошибка при отправке

$ #71 Original Message:(Protocol error)
# Ошибка протокола

$ #72 Original Message:(Multihop attempted)
# Была попытка перехода через несколько участков

$ #73 Original Message:(RFS specific error)
# Специфическая ошибка RFS

$ #74 Original Message:(Not a data message)
# Это не сообщение с данными

$ #75 Original Message:(Value too large for defined data type)
# Слишком большое значение для заданного типа данных

$ #76 Original Message:(Name not unique on network)
# Это имя не является уникальным в сети

$ #77 Original Message:(File descriptor in bad state)
# Файловый дескриптор находится в недопустимом состоянии

$ #78 Original Message:(Remote address changed)
# Изменился удалённый адрес

$ #79 Original Message:(Can not access a needed shared library)
# Необходимая разделяемая библиотека недоступна

$ #80 Original Message:(Accessing a corrupted shared library)
# Попытка доступа к повреждённой разделяемой библиотеке

$ #81 Original Message:(.lib section in a.out corrupted)
# Секция .lib в файле a.out повреждена

$ #82 Original Message:(Attempting to link in too many shared libraries)
# Попытка доступа к слишком большому числу разделяемых библиотек

$ #83 Original Message:(Cannot exec a shared library directly)
# Нельзя напрямую выполнить разделяемую библиотеку

$ #84 Original Message:(Illegal byte sequence)
# Недопустимая последовательность байтов

$ #85 Original Message:(Interrupted system call should be restarted)
# Прерванный системный вызов должен быть перезапущен

$ #86 Original Message:(Streams pipe error)
# Ошибка в потоковом конвейере

$ #87 Original Message:(Too many users)
# Слишком много пользователей

$ #88 Original Message:(Socket operation on non-socket)
# Операция с гнездом не на гнезде

$ #89 Original Message:(Destination address required)
# Требуется задать адрес назначения

$ #90 Original Message:(Message too long)
# Слишком длинное сообщение

$ #91 Original Message:(Protocol wrong type for socket)
# Неправильный тип протокола для гнезда

$ #92 Original Message:(Protocol not available)
# Протокол недоступен

$ #93 Original Message:(Protocol not supported)
# Протокол не поддерживается

$ #94 Original Message:(Socket type not supported)
# Тип гнезда не поддерживается

$ #95 Original Message:(Operation not supported on transport endpoint)
# Неподдерживаемая операция на транспортном окончании

$ #96 Original Message:(Protocol family not supported)
# Неподдерживаемое семейство протоколов

$ #97 Original Message:(Address family not supported by protocol)
# Протокол не поддерживает это семейство адресов

$ #98 Original Message:(Address already in use)
# Этот адрес уже используется

$ #99 Original Message:(Cannot assign requested address)
# Нельзя назначить заданный адрес

$ #100 Original Message:(Network is down)
# Сеть не работает

$ #101 Original Message:(Network is unreachable)
# Сеть недостижима

$ #102 Original Message:(Network dropped connection because of reset)
# Сеть оборвала соединение из-за сброса

$ #103 Original Message:(Software caused connection abort)
# Программное обеспечение вызвало обрыв соединения

$ #104 Original Message:(Connection reset by peer)
# Соединение сброшено другой строной

$ #105 Original Message:(No buffer space available)
# Исчерпано буферное пространство

$ #106 Original Message:(Transport endpoint is already connected)
# Транспортное окончание уже подсоединено

$ #107 Original Message:(Transport endpoint is not connected)
# Транспортное окончание ещё не подсоединено

$ #108 Original Message:(Cannot send after transport endpoint shutdown)
# Неврзможна отправка после закрытия транспортного окончания

$ #109 Original Message:(Too many references: cannot splice)
# Слишком много ссылок: невозможно сплести

$ #110 Original Message:(Connection timed out)
# Соединение обовано по таймауту

$ #111 Original Message:(Connection refused)
# Соединение отвергнуто

$ #112 Original Message:(Host is down)
# Удалённая машина не работает

$ #113 Original Message:(No route to host)
# Нет пути до машины

$ #114 Original Message:(Operation already in progress)
# Операция уже запущена

$ #115 Original Message:(Operation now in progress)
# Операция теперь запущена

$ #116 Original Message:(Stale NFS file handle)
# Устаревший дескриптор файла NFS

$ #117 Original Message:(Structure needs cleaning)
# Требуется очистка структуры

$ #118 Original Message:(Not a XENIX named type file)
# Это не именованный файл XENIX

$ #119 Original Message:(No XENIX semaphores available)
# Семафоры XENIX недоступны

$ #120 Original Message:(Is a named type file)
# Это именованный файл

$ #121 Original Message:(Remote I/O error)
# Удалённая ошибка ввода/вывода
