$set 3  #SignalList

$ #0 Original Message:(Unknown signal)
# Signal inconnu

$ #1 Original Message:(Hangup)
# Raccrochage

$ #2 Original Message:(Interrupt)
# Interruption

$ #3 Original Message:(Quit)
# Sortie

$ #4 Original Message:(Illegal instruction)
# Instruction illégale

$ #5 Original Message:(Trace/breakpoint trap)
# Trace/Point d'arrêt

$ #6 Original Message:(IOT trap/Abort)
# Piège IOT ou abandon

$ #7 Original Message:(Unused signal)
# Signal non utilisé

$ #8 Original Message:(Floating point exception)
# Exception numérique

$ #9 Original Message:(Killed)
# Tué

$ #10 Original Message:(User defined signal 1)
# Signal utilisateur 1

$ #11 Original Message:(Segmentation fault)
# Faute de segmentation

$ #12 Original Message:(User defined signal 2)
# Signal utilisateur 2

$ #13 Original Message:(Broken pipe)
# Tuyau rompu

$ #14 Original Message:(Alarm clock)
# Alarme

$ #15 Original Message:(Terminated)
# Terminé

$ #16 Original Message:(Stack fault)
# Faute de la pile

$ #17 Original Message:(Child exited)
# Un enfant a terminé

$ #18 Original Message:(Continued)
# Continue

$ #19 Original Message:(Stopped (signal))
# Stoppé (signal)

$ #20 Original Message:(Stopped)
# Stoppé

$ #21 Original Message:(Stopped (tty input))
# Stoppé (entrée sur terminal)

$ #22 Original Message:(Stopped (tty output))
# Stoppé (sortie sur le terminal)

$ #23 Original Message:(Urgent condition)
# Condition urgent

$ #24 Original Message:(CPU time limit exceeded)
# La limite du temps processeur a été dépassée

$ #25 Original Message:(File size limit exceeded)
# La limite de taille des fichiers a été dépassée

$ #26 Original Message:(Virtual time alarm)
# Alarme du temps virtuel

$ #27 Original Message:(Profile signal)
# Signal de profil

$ #28 Original Message:(Window size changed)
# La taille de la fenêtre a changé

$ #29 Original Message:(Possible I/O)
# E/S possible

$ #30 Original Message:(Unused signal)
# Signal non utilisé

$ #31 Original Message:(Unused signal)
# Signal non utilisé
