$set 1 #ErrorList

$ #0 Original Message:(Unknown error)
# Onbekende fout

$ #1 Original Message:(Operation not permitted)
# Dat is niet toegestaan

$ #2 Original Message:(No such file or directory)
# Ik kan dat bestand (of die bestandslijst) niet vinden

$ #3 Original Message:(No such process)
# Dat proces bestaat niet

$ #4 Original Message:(Interrupted system call)
# Systeemaanroep afgebroken

$ #5 Original Message:(I/O error)
# Fout bij invoer of uitvoer

$ #6 Original Message:(No such device or address)
# Dat apparaat of adres bestaat niet

$ #7 Original Message:(Arg list too long)
# Lijst met argumenten is te lang

$ #8 Original Message:(Exec format error)
# Dit is geen programma dat uitgevoerd kan worden

$ #9 Original Message:(Bad file number)
# Ondeugdelijke bestandsindex

$ #10 Original Message:(No child processes)
# Geen kind-processen

$ #11 Original Message:(Try again)
# Probeer nog maar eens ...

$ #12 Original Message:(Out of memory)
# Geen vrij geheugen meer

$ #13 Original Message:(Permission denied)
# Dat mag niet

$ #14 Original Message:(Bad address)
# Ongeldig adres

$ #15 Original Message:(Block device required)
# Dit vereist een apparaat met blokken

$ #16 Original Message:(Device or resource busy)
# Dat apparaat is bezig

$ #17 Original Message:(File exists)
# Dit bestand bestaat al

$ #18 Original Message:(Cross-device link)
# Harde verwijzing naar een ander apparaat kan niet

$ #19 Original Message:(No such device)
# Zo'n apparaat bestaat niet

$ #20 Original Message:(Not a directory)
# Dat is geen bestandslijst

$ #21 Original Message:(Is a directory)
# Dat is een bestandslijst

$ #22 Original Message:(Invalid argument)
# Ongeldige parameter

$ #23 Original Message:(File table overflow)
# De tabel met open bestanden is vol

$ #24 Original Message:(Too many open files)
# Te veel open bestanden

$ #25 Original Message:(Not a typewriter)
# Dat is geen (of niet jouw) toetsenbord

$ #26 Original Message:(Text file busy)
# Dat bestand is bezig

$ #27 Original Message:(File too large)
# Dat bestand is te groot

$ #28 Original Message:(No space left on device)
# Geen ruimte meer op het apparaat

$ #29 Original Message:(Illegal seek)
# Ongeldige zoekopdracht

$ #30 Original Message:(Read-only file system)
# Dit bestandssysteem kan alleen gelezen worden

$ #31 Original Message:(Too many links)
# Te veel verwijzingen

$ #32 Original Message:(Broken pipe)
# Gebroken pijp

$ #33 Original Message:(Math argument out of domain of func)
# Argument buiten het domein van een wiskundige functie

$ #34 Original Message:(Math result not representable)
# Wiskundig resultaat kan niet voorgesteld worden

$ #35 Original Message:(Resource deadlock would occur)
# Dat zou aanleiding tot een permanente impasse geven

$ #36 Original Message:(File name too long)
# Te lange bestandsnaam

$ #37 Original Message:(No record locks available)
# Geen regelsloten beschikbaar

$ #38 Original Message:(Function not implemented)
# Niet geomplementeerde functie

$ #39 Original Message:(Directory not empty)
# Bestandslijst niet leeg

$ #40 Original Message:(Too many symbolic links encountered)
# Te veel symbolische verwijzingen

$ #41 Original Message:(Operation would block)
# Die actie zou blokkeren

$ #42 Original Message:(No message of desired type)
# Er is geen boodschap van dit type

$ #43 Original Message:(Identifier removed)
# Naam (van een semafoor) verwijderd

$ #44 Original Message:(Channel number out of range)
# Kanaalnummer buiten het toegestane interval

$ #45 Original Message:(Level 2 not synchronized)
# Niveau 2 niet gesynchroniseerd

$ #46 Original Message:(Level 3 halted)
# Niveau 3 gestopt

$ #47 Original Message:(Level 3 reset)
# Niveau 3 herstart

$ #48 Original Message:(Link number out of range)
# Verwijzingsgetal buiten het toegestane interval

$ #49 Original Message:(Protocol driver not attached)
# Protocoldrijver niet aangesloten

$ #50 Original Message:(No CSI structure available)
# Geen CSI structuur beschikbaar

$ #51 Original Message:(Level 2 halted)
# Niveau 2 gestopt

$ #52 Original Message:(Invalid exchange)
# Ongeldige uitwisseling

$ #53 Original Message:(Invalid request descriptor)
# Ongeldige verzoekbeschijving

$ #54 Original Message:(Exchange full)
# Uitwisseling vol

$ #55 Original Message:(No anode)
# Geen anode

$ #56 Original Message:(Invalid request code)
# Ongeldige verzoekscode

$ #57 Original Message:(Invalid slot)
# Ongeldig vak

$ #58 Original Message:(File locking deadlock error)
# Permanente impasse bij bestandssloten

$ #59 Original Message:(Bad font file format)
# Slecht fontbestand

$ #60 Original Message:(Device not a stream)
# Dat apparaat is geen stroom

$ #61 Original Message:(No data available)
# Geen gegevens beschikbaar

$ #62 Original Message:(Timer expired)
# Wekker afgelopen

$ #63 Original Message:(Out of streams resources)
# Geen stromen meer beschikbaar

$ #64 Original Message:(Machine is not on the network)
# Machine heeft geen netwerkaansluiting

$ #65 Original Message:(Package not installed)
# Dat pakket is niet geonstalleerd

$ #66 Original Message:(Object is remote)
# Dat object is niet locaal

$ #67 Original Message:(Link has been severed)
# De verbinding is verbroken

$ #68 Original Message:(Advertise error)
# Adverteerfout

$ #69 Original Message:(Srmount error)
# Srmount fount

$ #70 Original Message:(Communication error on send)
# Communicatiefout by zenden

$ #71 Original Message:(Protocol error)
# Protocol fout

$ #72 Original Message:(Multihop attempted)
# Poging tot multihop

$ #73 Original Message:(RFS specific error)
# RFS-specifieke fout

$ #74 Original Message:(Not a data message)
# Geen bericht met gegevens

$ #75 Original Message:(Value too large for defined data type)
# Waarde te groot voor dit gegevenstype

$ #76 Original Message:(Name not unique on network)
# Deze naam is niet uniek op het netwerk

$ #77 Original Message:(File descriptor in bad state)
# Bestandsbeschijver is in een slechte toestand

$ #78 Original Message:(Remote address changed)
# Het niet-locale adres is gewijzigd

$ #79 Original Message:(Can not access a needed shared library)
# Ik kan een gedeelde bibliotheek die ik nodig heb niet vinden/lezen

$ #80 Original Message:(Accessing a corrupted shared library)
# De gedeelde bibliotheek die ik lees is beschadigd

$ #81 Original Message:(.lib section in a.out corrupted)
# De .lib sectie in a.out is beschadigd

$ #82 Original Message:(Attempting to link in too many shared libraries)
# Poging om te veel gedeelde bibliotheken mee te laden

$ #83 Original Message:(Cannot exec a shared library directly)
# Een gedeelde bibliotheek kan niet direct uitgevoerd worden

$ #84 Original Message:(Illegal byte sequence)
# Niet toegestane rij bytes

$ #85 Original Message:(Interrupted system call should be restarted)
# Onderbroken systeemaanroep moet herstart worden

$ #86 Original Message:(Streams pipe error)
# Fout bij stroompijp

$ #87 Original Message:(Too many users)
# Te veel gebruikers

$ #88 Original Message:(Socket operation on non-socket)
# Socket operatie op niet-socket

$ #89 Original Message:(Destination address required)
# Bestemmingsadres vereist

$ #90 Original Message:(Message too long)
# Boodschap te lang

$ #91 Original Message:(Protocol wrong type for socket)
# Protocol van verkeerd type voor socket

$ #92 Original Message:(Protocol not available)
# Protocol niet beschikbaar

$ #93 Original Message:(Protocol not supported)
# Protocol niet ondersteund

$ #94 Original Message:(Socket type not supported)
# Sockettype niet ondersteund

$ #95 Original Message:(Operation not supported on transport endpoint)
# Operatie niet ondersteund op het transporteindpunt

$ #96 Original Message:(Protocol family not supported)
# Protocolfamilie niet ondersteund

$ #97 Original Message:(Address family not supported by protocol)
# Adresfamilie niet ondersteund door protocol

$ #98 Original Message:(Address already in use)
# Adres al in gebruik

$ #99 Original Message:(Cannot assign requested address)
# Het gevraagde adres kan ik niet toewijzen

$ #100 Original Message:(Network is down)
# Het netwerk ligt plat

$ #101 Original Message:(Network is unreachable)
# Het netwerk is onbereikbaar

$ #102 Original Message:(Network dropped connection because of reset)
# Het network verbrak de verbinding vanwege een herstart

$ #103 Original Message:(Software caused connection abort)
# Verbinding door het programma afgekapt

$ #104 Original Message:(Connection reset by peer)
# Helaas, je partner heeft de verbinding verbroken

$ #105 Original Message:(No buffer space available)
# Geen opslagplaats beschikbaar

$ #106 Original Message:(Transport endpoint is already connected)
# Transporteindpunt is al verbonden

$ #107 Original Message:(Transport endpoint is not connected)
# Transporteindpunt is niet verbonden

$ #108 Original Message:(Cannot send after transport endpoint shutdown)
# Kan niet zenden na het uitschakelen van het transporteindpunt

$ #109 Original Message:(Too many references: cannot splice)
# Te veel referenties: kan niet splitsen

$ #110 Original Message:(Connection timed out)
# Verbinding verbroken wegens tijdsoverschrijding

$ #111 Original Message:(Connection refused)
# Verbinding geweigerd

$ #112 Original Message:(Host is down)
# Gastheer ligt plat

$ #113 Original Message:(No route to host)
# Geen pad naar gastheer

$ #114 Original Message:(Operation already in progress)
# Operatie al aan de gang

$ #115 Original Message:(Operation now in progress)
# Operatie nu aan de gang

$ #116 Original Message:(Stale NFS file handle)
# Oudbakken NFS bestandshandvat

$ #117 Original Message:(Structure needs cleaning)
# Structuur moet schoongemaakt worden

$ #118 Original Message:(Not a XENIX named type file)
# Geen (XENIX named)-type bestand

$ #119 Original Message:(No XENIX semaphores available)
# Geen XENIX semaforen beschikbaar

$ #120 Original Message:(Is a named type file)
# Is een bestand van named type

$ #121 Original Message:(Remote I/O error)
# Niet-locale fout bij invoer of uitvoer
