$set 11 #NetMisc

$ #AllPortsInUse Original Message:(All ports in use)
# Todos los puertos en uso

$ #ConnectTo Original Message:(connect to address)
# conexión a la dirección

$ #Trying Original Message:(Trying %s...\n)
# Intentando %s...\n

$ #SettingUpStderr Original Message:(write: setting up stderr)
# write: setting up stderr

$ #Accept Original Message:(accept)
# accept

$ #Getsockname Original Message:(getsockname)
# getsockname

$ #ProtocolFailure Original Message:(socket: protocol failure in circuit setup.)
# socket: fallo de protocolo (protocol failure in circuit setup.)

$ #RcmdSocket Original Message:(rcmd: socket)
# rcmd: socket

$ #RexecSocket Original Message:(rexec: socket)
# rexec: socket

$ #NetrcWrongPasswordMode Original Message:(Error - .netrc file not correct mode.\nRemove password or correct mode.)
# Error - el fichero .netrc no tiene el modo de acceso correcto.\nElimina el password o corrige el modo de acceso.

$ #UnknownNetrcOption Original Message:(Unknown .netrc option)
# Opción desconocida en .netrc

$ #ResolvIncorrectFormat Original Message:(resolv+: %s: \"%s\" command incorrectly formatted.\n)
# resolv+: %s: "%s" comando con formato incorrecto.\n

$ #ResolvInvalid Original Message:(resolv+: \"%s\" is an invalid keyword\n)
# resolv+: "%s" es un comando no válido\n

$ #ResolvValid Original Message:(resolv+: valid keywords are: %s, %s and %s\n)
# resolv+: comandos válidos son: %s, %s y %s\n

$ #Unrecognized Original Message:(resolv+: search order not specified or unrecognized keyword, host resolution will fail.\n)
# resolv+: orden de búsqueda no especificado o comando irreconocible, la búsqueda del host fallará.\n

$ #PossibleSpoof Original Message:(gethostbyaddr: %s != %u.%u.%u.%u, possible spoof attempt)
# gethostbyaddr: %s != %u.%u.%u.%u, possible spoof attempt

$ #SelectSetup Original Message:(rcmd: select (setting up stderr): )
# rcmd: select (inicializando stderr): 

$ #RcmdProtocolError Original Message:(select: protocol failure in circuit setup)
# select: fallo de protocolo en inicialización de circuito

$ #RhostLstat Original Message:(.rhosts lstat failed)
# .rhosts lstat failed

$ #RhostNotRegular Original Message:(.rhosts not regular file)
# .rhosts no es un fichero normal

$ #BadRhostsOwner Original Message:(bad .rhosts owner)
# propietario de .rhosts incorrecto

$ #RhostFstatFailed Original Message:(.rhosts fstat failed)
# .rhosts fstat failed

$ #RhostWritable Original Message:(.rhosts writeable by other than owner)
# .rhosts tiene permiso de escritura por usuarios distintos del propietario

$ #Name Original Message:(Name)
# Nombre

$ #Password Original Message:(Password)
# Password

$ #AskedForGot Original Message:(gethostby*.getanswer: asked for \"%s\", got \"%s\")
# gethostby*.getanswer: preguntó por "%s", consiguió "%s"

$ #AskedForGotCNAME Original Message:(gethostby*.getanswer: asked for \"%s\", got CNAME for \"%s\")
# gethostby*.getanswer: preguntó por "%s", consiguió CNAME para "%s"

$ #AskedForType Original Message:(gethostby*.getanswer: asked for type %d(%s), got %d(%s))
# gethostby*.getanswer: preguntó por tipo %d(%s), consiguió %d(%s)

