$set 10 #RpcMisc

$ #OutOfMemory Original Message:(out of memory)
# out of memory

$ #CannotRegister Original Message:(Cannot register service)
# Cannot register service

$ #FatalMarshall Original Message:(auth_none.c - Fatal marshalling problem)
# auth_none.c - Fatal marshalling problem

$ #FatalHeader Original Message:(clnt_raw.c - Fatal header serialization error.)
# clnt_raw.c - Fatal header serialization error.

$ #Hostname Original Message:(get_myaddress: gethostbyname)
# get_myaddress: gethostbyname

$ #Socket Original Message:(get_myaddress: socket)
# get_myaddress: socket

$ #Iface Original Message:(get_myaddress: ioctl (get interface configuration))
# get_myaddress: ioctl (get interface configuration)

$ #Ioctl Original Message:(get_myaddress: ioctl)
# get_myaddress: ioctl

$ #Getmaps Original Message:(pmap_getmaps rpc problem)
# pmap_getmaps rpc problem

$ #BConfig Original Message:(broadcast: ioctl (get interface configuration))
# broadcast: ioctl (get interface configuration)

$ #BFlags Original Message:(broadcast: ioctl (get interface flags))
# broadcast: ioctl (get interface flags)

$ #CantCreateBSocket Original Message:(Cannot create socket for broadcast rpc)
# Cannot create socket for broadcast rpc

$ #CantSetBOption Original Message:(Cannot set socket option SO_BROADCAST)
# Cannot set socket option SO_BROADCAST

$ #CantSendBPacket Original Message:(Cannot send broadcast packet)
# Cannot send broadcast packet

$ #BSelectProblem Original Message:(Broadcast select problem)
# Broadcast select problem

$ #CantReceiveBReply Original Message:(Cannot receive reply to broadcast)
# Cannot receive reply to broadcast

$ #BDesProblem Original Message:(Broadcast deserialization problem)
# Broadcast deserialization problem

$ #BadAuthLen Original Message:(bad auth_len gid %d str %d auth %d\n)
# bad auth_len gid %d str %d auth %d\n

$ #SelectFailed Original Message:(svc_run: - select failed)
# svc_run: - select failed

$ #CantReassignProc Original Message:(can't reassign procedure number %d\n)
# can't reassign procedure number %d\n

$ #CantCreateServer Original Message:(couldn't create an rpc server)
# couldn't create an rpc server

$ #CantRegisterProg Original Message:(couldn't register prog %d vers %d\n)
# couldn't register prog %d vers %d\n

$ #TroubleReplying Original Message:(trouble replying to prog %d\n)
# trouble replying to prog %d\n

$ #NeverRegisteredProg Original Message:(never registered prog %d\n)
# never registered prog %d\n

$ #TcpSocketCreateProblem Original Message:(svc_tcp.c - tcp socket creation problem)
# svc_tcp.c - tcp socket creation problem

$ #CantGetNameListen Original Message:(svctcp_.c - cannot getsockname or listen)
# svctcp_.c - cannot getsockname or listen

$ #UdpSocketCreateProblem Original Message:(svcudp_create: socket creation problem)
# svcudp_create: socket creation problem

$ #UdpCantGetName Original Message:(svcudp_create - cannot getsockname)
# svcudp_create - cannot getsockname

$ #CacheAlreadyEnabled Original Message:(enablecache: cache already enabled)
# enablecache: cache already enabled

$ #CantAllocateCache Original Message:(enablecache: could not allocate cache)
# enablecache: could not allocate cache

$ #CantAllocateCacheData Original Message:(enablecache: could not allocate cache data)
# enablecache: could not allocate cache data

$ #CantAllocateCacheFifo Original Message:(enablecache: could not allocate cache fifo)
# enablecache: could not allocate cache fifo

$ #VictimNotFound Original Message:(cache_set: victim not found)
# cache_set: victim not found

$ #VictimAllocFailed Original Message:(cache_set: victim alloc failed)
# cache_set: victim alloc failed

$ #CantAllocateRpcBuffer Original Message:(cache_set: could not allocate new rpc_buffer)
# cache_set: could not allocate new rpc_buffer
