/* Copyright (C) 1995 Free Software Foundation, Inc.
This file is part of the Linux Library

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <ansidecl.h>
#include <math.h>
#include "ieee854.h"

/* Return X with its signed changed to Y's.  */
long double
DEFUN(__copysignl, (x, y), long double x AND long double y)
{
  union ieee854_double *ux = (union ieee854_double *) &x;
  union ieee854_double *uy = (union ieee854_double *) &y;

  ux->ieee.negative = uy->ieee.negative;

  return x;
}

#include <gnu-stabs.h>
#ifdef weak_alias
weak_alias (__copysignl, copysignl);
#endif
