/* Copyright (C) 1991 Free Software Foundation, Inc.
This file is part of the GNU C Library.

The GNU C Library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

The GNU C Library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with the GNU C Library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.  */

#include <ansidecl.h>
#include <errno.h>
#include <stddef.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#ifdef __linux__
#include <sys/vfs.h>
#include <linux/socket.h>
#include <linux/nfs_fs.h>
#endif


/* Get file-specific information about descriptor FD.  */
long int
DEFUN(__fpathconf, (fd, name), int fd AND int name)
{
  if (fd < 0)
    {
      errno = EBADF;
      return -1;
    }

  switch (name)
    {
    default:
      errno = EINVAL;
      return -1;

    case _PC_LINK_MAX:
#ifdef	LINK_MAX
      return LINK_MAX;
#else
      errno = ENOSYS;
      return -1;
#endif

    case _PC_MAX_CANON:
#ifdef	MAX_CANON
      return MAX_CANON;
#else
      errno = ENOSYS;
      return -1;
#endif

    case _PC_MAX_INPUT:
#ifdef	MAX_INPUT
      return MAX_INPUT;
#else
      errno = ENOSYS;
      return -1;
#endif

    case _PC_NAME_MAX:
#ifdef __linux__
      {
        struct statfs buf;
	if (__fstatfs (fd, &buf) < 0) return -1;
	else return buf.f_namelen;
      }
#else
#ifdef	NAME_MAX
      return NAME_MAX;
#else
      errno = ENOSYS;
      return -1;
#endif
#endif

    case _PC_PATH_MAX:
#ifdef	PATH_MAX
      return PATH_MAX;
#else
      errno = ENOSYS;
      return -1;
#endif

    case _PC_PIPE_BUF:
#ifdef	PIPE_BUF
      return PIPE_BUF;
#else
      errno = ENOSYS;
      return -1;
#endif

    case _PC_CHOWN_RESTRICTED:
#ifdef	_POSIX_CHOWN_RESTRICTED
# ifdef __linux__
      { 
	struct statfs buf;
	if (__fstatfs (fd, &buf) < 0) return -1;
	else {
	  if (buf.f_type == NFS_SUPER_MAGIC) {
	    errno = EINVAL;
	    return -1;
	  } else
	    return _POSIX_CHOWN_RESTRICTED;
	}
      }
# else
      return _POSIX_CHOWN_RESTRICTED;
# endif
#else
      return -1;
#endif

    case _PC_NO_TRUNC:
#ifdef	_POSIX_NO_TRUNC
      return _POSIX_NO_TRUNC;
#else
      return -1;
#endif

    case _PC_VDISABLE:
#ifdef	_POSIX_VDISABLE
      return _POSIX_VDISABLE;
#else
      return -1;
#endif
    }

  errno = ENOSYS;
  return -1;
}

#include <gnu-stabs.h>
#ifdef weak_alias
weak_alias (__fpathconf, fpathconf);
#endif
