/*
 * Please do not edit this file.
 * It was generated using rpcgen.
 */

#include <rpcsvc/yppasswd.h>
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user or with the express written consent of
 * Sun Microsystems, Inc.
 *
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 *
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 *
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 *
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 *
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */
/*
 * Copyright (c) 1986, 1990 by Sun Microsystems, Inc.
 */

/* from @(#)yppasswd.x	1.3 91/03/11 TIRPC 1.0 */

bool_t
xdr_passwd(XDR *xdrs, struct passwd *objp)
{
  
  if (!xdr_string(xdrs, &objp->pw_name, ~0))
    {
      return (FALSE);
    }
  if (!xdr_string(xdrs, &objp->pw_passwd, ~0))
    {
      return (FALSE);
    }
  if (!xdr_int(xdrs, &objp->pw_uid))
    {
      return (FALSE);
    }
  if (!xdr_int(xdrs, &objp->pw_gid))
    {
      return (FALSE);
    }
  if (!xdr_string(xdrs, &objp->pw_gecos, ~0))
    {
      return (FALSE);
    }
  if (!xdr_string(xdrs, &objp->pw_dir, ~0))
    {
      return (FALSE);
    }
  if (!xdr_string(xdrs, &objp->pw_shell, ~0))
    {
      return (FALSE);
    }
  return (TRUE);
}

bool_t
xdr_yppasswd(XDR *xdrs, struct yppasswd *objp)
{
  
  if (!xdr_string(xdrs, &objp->oldpass, ~0))
    {
      return (FALSE);
    }
  if (!xdr_passwd(xdrs, &objp->newpw))
    {
      return (FALSE);
    }
  return (TRUE);
}
