$set 3  #SignalList

$ #0 Original Message:(Unknown signal)
# Неизвестный сигнал

$ #1 Original Message:(Hangup)
# Зависание

$ #2 Original Message:(Interrupt)
# Прерывание

$ #3 Original Message:(Quit)
# Выход

$ #4 Original Message:(Illegal instruction)
# Запрещенная инструкция

$ #5 Original Message:(Trace/breakpoint trap)
# Прерывание на точке останова или при трассировке

$ #6 Original Message:(IOT trap/Abort)
# IOT trap/Abort

$ #7 Original Message:(Bus Error)
# Ошибка шины

$ #8 Original Message:(Floating point exception)
# Исключительная ситуация при операции с плавающей точкой

$ #9 Original Message:(Killed)
# Убит

$ #10 Original Message:(User defined signal 1)
# Задаваемый пользователем сигнал #1

$ #11 Original Message:(Segmentation fault)
# Ошибка сегментации

$ #12 Original Message:(User defined signal 2)
# Задаваемый пользователем сигнал #2

$ #13 Original Message:(Broken pipe)
# Обованный конвейер

$ #14 Original Message:(Alarm clock)
# Сигнал от будильника

$ #15 Original Message:(Terminated)
# Прекращён

$ #16 Original Message:(Stack fault)
# Ошибка стека

$ #17 Original Message:(Child exited)
# Окончание дочернего процесса

$ #18 Original Message:(Continued)
# Продолжение

$ #19 Original Message:(Stopped (signal))
# Остановлен (signal)

$ #20 Original Message:(Stopped)
# Остановлен

$ #21 Original Message:(Stopped (tty input))
# Остановлен (tty input)

$ #22 Original Message:(Stopped (tty output))
# Остановлен (tty output)

$ #23 Original Message:(Urgent condition)
# Экстренное состояние

$ #24 Original Message:(CPU time limit exceeded)
# Превышен лимит времени на CPU

$ #25 Original Message:(File size limit exceeded)
# Превышен лимит на размер файла

$ #26 Original Message:(Virtual time alarm)
# Сигнал от виртуального будильника

$ #27 Original Message:(Profile signal)
# Сигнал профилирования

$ #28 Original Message:(Window size changed)
# Изменён размер окна

$ #29 Original Message:(Possible I/O)
# Возможный ввод/вывод

$ #30 Original Message:(Power Failure)
# Сбой питания

$ #31 Original Message:(Unused signal)
# Неиспользуемый сигнал
