$set 5 #HerrorList

$ #1 Original Message:(Error 0)
# Ошибка 0

$ #2 Original Message:(Unknown host)
# Неизвестная машина

$ #3 Original Message:(Host name lookup failure)
# Ошибка поиска имени машины

$ #4 Original Message:(Unknown server error)
# Неизвестная ошибка сервера

$ #5 Original Message:(No address associated with name)
# Этому имени не соответствует никакого адреса

