
#define ErrorListSet	0x1
#define ErrorList0	0x1
#define ErrorList1	0x2
#define ErrorList2	0x3
#define ErrorList3	0x4
#define ErrorList4	0x5
#define ErrorList5	0x6
#define ErrorList6	0x7
#define ErrorList7	0x8
#define ErrorList8	0x9
#define ErrorList9	0xa
#define ErrorList10	0xb
#define ErrorList11	0xc
#define ErrorList12	0xd
#define ErrorList13	0xe
#define ErrorList14	0xf
#define ErrorList15	0x10
#define ErrorList16	0x11
#define ErrorList17	0x12
#define ErrorList18	0x13
#define ErrorList19	0x14
#define ErrorList20	0x15
#define ErrorList21	0x16
#define ErrorList22	0x17
#define ErrorList23	0x18
#define ErrorList24	0x19
#define ErrorList25	0x1a
#define ErrorList26	0x1b
#define ErrorList27	0x1c
#define ErrorList28	0x1d
#define ErrorList29	0x1e
#define ErrorList30	0x1f
#define ErrorList31	0x20
#define ErrorList32	0x21
#define ErrorList33	0x22
#define ErrorList34	0x23
#define ErrorList35	0x24
#define ErrorList36	0x25
#define ErrorList37	0x26
#define ErrorList38	0x27
#define ErrorList39	0x28
#define ErrorList40	0x29
#define ErrorList41	0x2a
#define ErrorList42	0x2b
#define ErrorList43	0x2c
#define ErrorList44	0x2d
#define ErrorList45	0x2e
#define ErrorList46	0x2f
#define ErrorList47	0x30
#define ErrorList48	0x31
#define ErrorList49	0x32
#define ErrorList50	0x33
#define ErrorList51	0x34
#define ErrorList52	0x35
#define ErrorList53	0x36
#define ErrorList54	0x37
#define ErrorList55	0x38
#define ErrorList56	0x39
#define ErrorList57	0x3a
#define ErrorList58	0x3b
#define ErrorList59	0x3c
#define ErrorList60	0x3d
#define ErrorList61	0x3e
#define ErrorList62	0x3f
#define ErrorList63	0x40
#define ErrorList64	0x41
#define ErrorList65	0x42
#define ErrorList66	0x43
#define ErrorList67	0x44
#define ErrorList68	0x45
#define ErrorList69	0x46
#define ErrorList70	0x47
#define ErrorList71	0x48
#define ErrorList72	0x49
#define ErrorList73	0x4a
#define ErrorList74	0x4b
#define ErrorList75	0x4c
#define ErrorList76	0x4d
#define ErrorList77	0x4e
#define ErrorList78	0x4f
#define ErrorList79	0x50
#define ErrorList80	0x51
#define ErrorList81	0x52
#define ErrorList82	0x53
#define ErrorList83	0x54
#define ErrorList84	0x55
#define ErrorList85	0x56
#define ErrorList86	0x57
#define ErrorList87	0x58
#define ErrorList88	0x59
#define ErrorList89	0x5a
#define ErrorList90	0x5b
#define ErrorList91	0x5c
#define ErrorList92	0x5d
#define ErrorList93	0x5e
#define ErrorList94	0x5f
#define ErrorList95	0x60
#define ErrorList96	0x61
#define ErrorList97	0x62
#define ErrorList98	0x63
#define ErrorList99	0x64
#define ErrorList100	0x65
#define ErrorList101	0x66
#define ErrorList102	0x67
#define ErrorList103	0x68
#define ErrorList104	0x69
#define ErrorList105	0x6a
#define ErrorList106	0x6b
#define ErrorList107	0x6c
#define ErrorList108	0x6d
#define ErrorList109	0x6e
#define ErrorList110	0x6f
#define ErrorList111	0x70
#define ErrorList112	0x71
#define ErrorList113	0x72
#define ErrorList114	0x73
#define ErrorList115	0x74
#define ErrorList116	0x75
#define ErrorList117	0x76
#define ErrorList118	0x77
#define ErrorList119	0x78
#define ErrorList120	0x79
#define ErrorList121	0x7a

#define UnknownErrorSet	0x2
#define UnknownError1	0x1
#define UnknownError2	0x2

#define SignalListSet	0x3
#define SignalList0	0x1
#define SignalList1	0x2
#define SignalList2	0x3
#define SignalList3	0x4
#define SignalList4	0x5
#define SignalList5	0x6
#define SignalList6	0x7
#define SignalList7	0x8
#define SignalList8	0x9
#define SignalList9	0xa
#define SignalList10	0xb
#define SignalList11	0xc
#define SignalList12	0xd
#define SignalList13	0xe
#define SignalList14	0xf
#define SignalList15	0x10
#define SignalList16	0x11
#define SignalList17	0x12
#define SignalList18	0x13
#define SignalList19	0x14
#define SignalList20	0x15
#define SignalList21	0x16
#define SignalList22	0x17
#define SignalList23	0x18
#define SignalList24	0x19
#define SignalList25	0x1a
#define SignalList26	0x1b
#define SignalList27	0x1c
#define SignalList28	0x1d
#define SignalList29	0x1e
#define SignalList30	0x1f
#define SignalList31	0x20

#define UnknownSignalSet	0x4
#define UnknownSignal0	0x1
#define UnknownSignal1	0x2

#define HerrorListSet	0x5
#define HerrorList1	0x1
#define HerrorList2	0x2
#define HerrorList3	0x3
#define HerrorList4	0x4
#define HerrorList5	0x5

#define GetoptSet	0x6
#define GetoptAmbiguous	0x1
#define GetoptNoArgumentsAllowed1	0x2
#define GetoptNoArgumentsAllowed2	0x3
#define GetoptRequiresArgument1	0x4
#define GetoptRequiresArgument2	0x5
#define GetoptUnrecognized1	0x6
#define GetoptUnrecognized2	0x7
#define GetoptIllegal	0x8

#define AuthListSet	0x7
#define AuthList1	0x1
#define AuthList2	0x2
#define AuthList3	0x3
#define AuthList4	0x4
#define AuthList5	0x5
#define AuthList6	0x6
#define AuthList7	0x7
#define AuthList8	0x8

#define RpcErrListSet	0x8
#define RpcErrList1	0x1
#define RpcErrList2	0x2
#define RpcErrList3	0x3
#define RpcErrList4	0x4
#define RpcErrList5	0x5
#define RpcErrList6	0x6
#define RpcErrList7	0x7
#define RpcErrList8	0x8
#define RpcErrList9	0x9
#define RpcErrList10	0xa
#define RpcErrList11	0xb
#define RpcErrList12	0xc
#define RpcErrList13	0xd
#define RpcErrList14	0xe
#define RpcErrList15	0xf
#define RpcErrList16	0x10
#define RpcErrList17	0x11
#define RpcErrList18	0x12
#define RpcErrListUnknown	0x13

#define ClntMiscSet	0x9
#define ClntMiscErrno	0x1
#define ClntMiscLowHigh	0x2
#define ClntMiscWhy	0x3
#define ClntMiscUnknownAuth	0x4

#define RpcMiscSet	0xa
#define RpcMiscOutOfMemory	0x1
#define RpcMiscCannotRegister	0x2
#define RpcMiscFatalMarshall	0x3
#define RpcMiscFatalHeader	0x4
#define RpcMiscHostname	0x5
#define RpcMiscSocket	0x6
#define RpcMiscIface	0x7
#define RpcMiscIoctl	0x8
#define RpcMiscGetmaps	0x9
#define RpcMiscBConfig	0xa
#define RpcMiscBFlags	0xb
#define RpcMiscCantCreateBSocket	0xc
#define RpcMiscCantSetBOption	0xd
#define RpcMiscCantSendBPacket	0xe
#define RpcMiscBSelectProblem	0xf
#define RpcMiscCantReceiveBReply	0x10
#define RpcMiscBDesProblem	0x11
#define RpcMiscBadAuthLen	0x12
#define RpcMiscSelectFailed	0x13
#define RpcMiscCantReassignProc	0x14
#define RpcMiscCantCreateServer	0x15
#define RpcMiscCantRegisterProg	0x16
#define RpcMiscTroubleReplying	0x17
#define RpcMiscNeverRegisteredProg	0x18
#define RpcMiscTcpSocketCreateProblem	0x19
#define RpcMiscCantGetNameListen	0x1a
#define RpcMiscUdpSocketCreateProblem	0x1b
#define RpcMiscUdpCantGetName	0x1c
#define RpcMiscCacheAlreadyEnabled	0x1d
#define RpcMiscCantAllocateCache	0x1e
#define RpcMiscCantAllocateCacheData	0x1f
#define RpcMiscCantAllocateCacheFifo	0x20
#define RpcMiscVictimNotFound	0x21
#define RpcMiscVictimAllocFailed	0x22
#define RpcMiscCantAllocateRpcBuffer	0x23

#define NetMiscSet	0xb
#define NetMiscAllPortsInUse	0x1
#define NetMiscConnectTo	0x2
#define NetMiscTrying	0x3
#define NetMiscSettingUpStderr	0x4
#define NetMiscAccept	0x5
#define NetMiscGetsockname	0x6
#define NetMiscProtocolFailure	0x7
#define NetMiscRcmdSocket	0x8
#define NetMiscRexecSocket	0x9
#define NetMiscNetrcWrongPasswordMode	0xa
#define NetMiscUnknownNetrcOption	0xb
#define NetMiscResolvIncorrectFormat	0xc
#define NetMiscResolvInvalid	0xd
#define NetMiscResolvValid	0xe
#define NetMiscUnrecognized	0xf
#define NetMiscPossibleSpoof	0x10
#define NetMiscSelectSetup	0x11
#define NetMiscRcmdProtocolError	0x12
#define NetMiscRhostLstat	0x13
#define NetMiscRhostNotRegular	0x14
#define NetMiscBadRhostsOwner	0x15
#define NetMiscRhostFstatFailed	0x16
#define NetMiscRhostWritable	0x17
#define NetMiscName	0x18
#define NetMiscPassword	0x19
#define NetMiscAskedForGot	0x1a
#define NetMiscAskedForGotCNAME	0x1b
#define NetMiscAskedForType	0x1c
