$set 10 #RpcMisc

$ #OutOfMemory Original Message:(out of memory)
# Plus de mémoire

$ #CannotRegister Original Message:(Cannot register service)
# Ne peut pas enregistrer le service

$ #FatalMarshall Original Message:(auth_none.c - Fatal marshalling problem)
# auth_none.c - Problème fatal de maintien de l'ordre

$ #FatalHeader Original Message:(clnt_raw.c - Fatal header serialization error.)
# clnt_raw.c - Erreur fatale de sérialisation de l'entête.

$ #Hostname Original Message:(get_myaddress: gethostbyname)
# get_myaddress: gethostbyname

$ #Socket Original Message:(get_myaddress: socket)
# get_myaddress: socket

$ #Iface Original Message:(get_myaddress: ioctl (get interface configuration))
# get_myaddress: ioctl (demande de la configuration de l'interface)

$ #Ioctl Original Message:(get_myaddress: ioctl)
# get_myaddress: ioctl

$ #Getmaps Original Message:(pmap_getmaps rpc problem)
# pmap_getmaps: problème rpc

$ #BConfig Original Message:(broadcast: ioctl (get interface configuration))
# broadcast: ioctl (demande de la configuration de l'interface)

$ #BFlags Original Message:(broadcast: ioctl (get interface flags))
# broadcast: ioctl (demande des drapeaux de l'interface)

$ #CantCreateBSocket Original Message:(Cannot create socket for broadcast rpc)
# Ne peut créer un socket pour une diffusion rpc

$ #CantSetBOption Original Message:(Cannot set socket option SO_BROADCAST)
# Ne peut mettre l'option socket SO_BROADCAST

$ #CantSendBPacket Original Message:(Cannot send broadcast packet)
# Ne peut envoyer un paquet de diffusion

$ #BSelectProblem Original Message:(Broadcast select problem)
# Problème de selection de diffusion

$ #CantReceiveBReply Original Message:(Cannot receive reply to broadcast)
# Ne peut recevoir la réponse à une diffusion

$ #BDesProblem Original Message:(Broadcast deserialization problem)
# Problème de sérialisation d'un diffusion

$ #BadAuthLen Original Message:(bad auth_len gid %d str %d auth %d\n)
# mauvais auth_len gid %d str %d auth %d\n

$ #SelectFailed Original Message:(svc_run: - select failed)
# svc_run: - échec de select

$ #CantReassignProc Original Message:(can't reassign procedure number %d\n)
# Ne peut réassigner la précedure numéro %d\n

$ #CantCreateServer Original Message:(couldn't create an rpc server)
# ne peut créer un serveur rpc

$ #CantRegisterProg Original Message:(couldn't register prog %d vers %d\n)
# ne peut enregistrer prog %d vers %d\n

$ #TroubleReplying Original Message:(trouble replying to prog %d\n)
# trouble en répondant au prog %d\n

$ #NeverRegisteredProg Original Message:(never registered prog %d\n)
# prog %d jamais enregistré\n

$ #TcpSocketCreateProblem Original Message:(svc_tcp.c - tcp socket creation problem)
# svc_tcp.c - problème de création de socket tcp

$ #CantGetNameListen Original Message:(svctcp_.c - cannot getsockname or listen)
# svctcp_.c - problème avec listen ou getsockname

$ #UdpSocketCreateProblem Original Message:(svcudp_create: socket creation problem)
# svcudp_create: problème de création de socket

$ #UdpCantGetName Original Message:(svcudp_create - cannot getsockname)
# svcudp_create - problème avec getsockname

$ #CacheAlreadyEnabled Original Message:(enablecache: cache already enabled)
# enablecache: cache déjà permis

$ #CantAllocateCache Original Message:(enablecache: could not allocate cache)
# enablecache: ne peut allouer un cache

$ #CantAllocateCacheData Original Message:(enablecache: could not allocate cache data)
# enablecache: ne peut allouer un cache de données

$ #CantAllocateCacheFifo Original Message:(enablecache: could not allocate cache fifo)
# enablecache: ne peut allouer un cache de fifo

$ #VictimNotFound Original Message:(cache_set: victim not found)
# cache_set: victime non trouvée

$ #VictimAllocFailed Original Message:(cache_set: victim alloc failed)
# cache_set: échec de l'allocation de la victime

$ #CantAllocateRpcBuffer Original Message:(cache_set: could not allocate new rpc_buffer)
# cache_set: ne peut pas allouer un nouveau rpc_buffer
