$set 8 #RpcErrList

$ #1 Original Message:(RPC: Success)
# RPC: Успех

$ #2 Original Message:(RPC: Can't encode arguments)
# RPC: Нельзя закодировать аргументы

$ #3 Original Message:(RPC: Can't decode result)
# RPC: Нельзя раскодировать результат

$ #4 Original Message:(RPC: Unable to send)
# RPC: Отправка невозможна

$ #5 Original Message:(RPC: Unable to receive)
# RPC: Приём невозможен

$ #6 Original Message:(RPC: Timed out)
# RPC: Время истекло

$ #7 Original Message:(RPC: Incompatible versions of RPC)
# RPC: Несовместимая версия RPC

$ #8 Original Message:(RPC: Authentication error)
# RPC: Ошибка удостоверения подлинности

$ #9 Original Message:(RPC: Program unavailable)
# RPC: Программа недоступна

$ #10 Original Message:(RPC: Program/version mismatch)
# RPC: Программа/версия не совпадают

$ #11 Original Message:(RPC: Procedure unavailable)
# RPC: Процедура недоступна

$ #12 Original Message:(RPC: Server can't decode arguments)
# RPC: Сервер не может раскодировать аргументы

$ #13 Original Message:(RPC: Remote system error)
# RPC: Удалённая системная ошибка

$ #14 Original Message:(RPC: Unknown host)
# RPC: Неизвестная машина

$ #15 Original Message:(RPC: Port mapper failure)
# RPC: Ошибка Port mapper

$ #16 Original Message:(RPC: Program not registered)
# RPC: Программа не зарегистрирована

$ #17 Original Message:(RPC: Failed (unspecified error))
# RPC: Окончилось неудачно (неуказанная ошибка)

$ #18 Original Message:(RPC: Unknown protocol)
# RPC: Неизвестный протокол

$ #Unknown Original Message:(RPC: (unknown error code))
# RPC: (неизвестный код ошибки)
