$set 7 #AuthList

$ #1 Original Message:(Authentication OK)
# Autenticación correcta

$ #2 Original Message:(Invalid client credential)
# Credencial de cliente no válida

$ #3 Original Message:(Server rejected credential)
# Credential rechazada por el servidor

$ #4 Original Message:(Invalid client verifier)
# Verificador de cliente no válido

$ #5 Original Message:(Server rejected verifier)
# Verificador rechazado por el servidor

$ #6 Original Message:(Client credential too weak)
# Credential de cliente demasiado débil

$ #7 Original Message:(Invalid server verifier)
# Verificador de servidor no válido

$ #8 Original Message:(Failed (unspecified error))
# Fallo (error no especificado)

