$set 1 #ErrorList

$ #0 Original Message:(Unknown error)
# Error desconocido

$ #1 Original Message:(Operation not permitted)
# Operación no permitida

$ #2 Original Message:(No such file or directory)
# Fichero o directorio inexistente

$ #3 Original Message:(No such process)
# Proceso inexistente

$ #4 Original Message:(Interrupted system call)
# Llamada al sistema interrumpida

$ #5 Original Message:(I/O error)
# Error de entrada/salida

$ #6 Original Message:(No such device or address)
# Dispositivo o dirección inexistente

$ #7 Original Message:(Arg list too long)
# Lista de argumentos demasiado larga

$ #8 Original Message:(Exec format error)
# Error en el formato del ejecutable

$ #9 Original Message:(Bad file number)
# Número de fichero mal

$ #10 Original Message:(No child processes)
# No hay procesos hijos

$ #11 Original Message:(Try again)
# Prueba otra vez

$ #12 Original Message:(Out of memory)
# No hay memoria suficiente

$ #13 Original Message:(Permission denied)
# Permiso denegado

$ #14 Original Message:(Bad address)
# Dirección mal

$ #15 Original Message:(Block device required)
# Dispositivo de bloque requerido

$ #16 Original Message:(Device or resource busy)
# Dispositivo o recurso ocupado

$ #17 Original Message:(File exists)
# El fichero ya existe

$ #18 Original Message:(Cross-device link)
# Cross-device link

$ #19 Original Message:(No such device)
# No existe el dispositivo

$ #20 Original Message:(Not a directory)
# No es un directorio

$ #21 Original Message:(Is a directory)
# Es un directorio

$ #22 Original Message:(Invalid argument)
# Argumento no válido

$ #23 Original Message:(File table overflow)
# Desbordamiento en la tabla de ficheros

$ #24 Original Message:(Too many open files)
# Demasiados ficheros abiertos

$ #25 Original Message:(Not a typewriter)
# Not a typewriter

$ #26 Original Message:(Text file busy)
# Fichero ocupado

$ #27 Original Message:(File too large)
# Fichero demasiado grande

$ #28 Original Message:(No space left on device)
# No queda espacio en el dispositivo

$ #29 Original Message:(Illegal seek)
# Seek ilegal

$ #30 Original Message:(Read-only file system)
# sistema de ficheros de sólo lectura

$ #31 Original Message:(Too many links)
# Demasiados enlaces

$ #32 Original Message:(Broken pipe)
# tubería (pipe) rota

$ #33 Original Message:(Math argument out of domain of func)
# Argumento matemático fuera del dominio de la función

$ #34 Original Message:(Math result not representable)
# Resultado matemático no representable

$ #35 Original Message:(Resource deadlock would occur)
# Posibilidad de un bloqueo (deadlock) de recursos

$ #36 Original Message:(File name too long)
# Nombre de fichero demasiado largo

$ #37 Original Message:(No record locks available)
# No quedan bloqueos (locks) de registro

$ #38 Original Message:(Function not implemented)
# Función no implementada

$ #39 Original Message:(Directory not empty)
# Directorio no vacío

$ #40 Original Message:(Too many symbolic links encountered)
# Demasiados enlaces (links) simbólicos

$ #41 Original Message:(Operation would block)
# La operación podría causar un bloqueo

$ #42 Original Message:(No message of desired type)
# No hay un mensaje del tipo deseado

$ #43 Original Message:(Identifier removed)
# Identificador eliminado

$ #44 Original Message:(Channel number out of range)
# Número de canal fuera de rango

$ #45 Original Message:(Level 2 not synchronized)
# Nivel 2 no sincronizado

$ #46 Original Message:(Level 3 halted)
# Nivel 3 parado

$ #47 Original Message:(Level 3 reset)
# Nivel 3 reinicializado

$ #48 Original Message:(Link number out of range)
# Número de enlace fuera de rango

$ #49 Original Message:(Protocol driver not attached)
# Controlador de protocolo no conectado

$ #50 Original Message:(No CSI structure available)
# No estructura CSI disponible

$ #51 Original Message:(Level 2 halted)
# Nivel 2 parado

$ #52 Original Message:(Invalid exchange)
# Intercambio no válido

$ #53 Original Message:(Invalid request descriptor)
# Descriptor de petición (request descriptor) no válido

$ #54 Original Message:(Exchange full)
# Exchange full

$ #55 Original Message:(No anode)
# No anode

$ #56 Original Message:(Invalid request code)
# Invalid request code

$ #57 Original Message:(Invalid slot)
# Ranura no válida

$ #58 Original Message:(File locking deadlock error)
# Error por bloqueo (deadlock) de fichero

$ #59 Original Message:(Bad font file format)
# Formato de fuente incorrecto

$ #60 Original Message:(Device not a stream)
# El dispositivo ne es un stream

$ #61 Original Message:(No data available)
# No hay datos disponibles

$ #62 Original Message:(Timer expired)
# Temporizador expirado

$ #63 Original Message:(Out of streams resources)
# No quedan streams disponibles

$ #64 Original Message:(Machine is not on the network)
# La máquina no está en la red

$ #65 Original Message:(Package not installed)
# Paquete (Package) no instalado

$ #66 Original Message:(Object is remote)
# El objeto es remoto

$ #67 Original Message:(Link has been severed)
# El enlace ha sido cortado

$ #68 Original Message:(Advertise error)
# Advertise error

$ #69 Original Message:(Srmount error)
# Srmount error

$ #70 Original Message:(Communication error on send)
# Error de comunicacion al enviar (send)

$ #71 Original Message:(Protocol error)
# Error de protocolo

$ #72 Original Message:(Multihop attempted)
# Intentado multihop 

$ #73 Original Message:(RFS specific error)
# error específico de RFS

$ #74 Original Message:(Not a data message)
# No es un mensaje de datos

$ #75 Original Message:(Value too large for defined data type)
# Valor demasiado grande para el tipo de dato definido

$ #76 Original Message:(Name not unique on network)
# Nombre no único en la red

$ #77 Original Message:(File descriptor in bad state)
# Descriptor de fichero en mal estado

$ #78 Original Message:(Remote address changed)
# Dirección remota cambiada

$ #79 Original Message:(Can not access a needed shared library)
# No puedo acceder a una biblioteca compartida (shared library) necesaria

$ #80 Original Message:(Accessing a corrupted shared library)
# Accediendo a una biblioteca compartida (shared library) en mal estado

$ #81 Original Message:(.lib section in a.out corrupted)
# .lib sección in a.out en mal estado

$ #82 Original Message:(Attempting to link in too many shared libraries)
# Intentando enlazar con demasiadas bibliotecas compartidas (shared libraries)

$ #83 Original Message:(Cannot exec a shared library directly)
# No puedo ejecutar una biblioteca compartida (shared library) directamente

$ #84 Original Message:(Illegal byte sequence)
# Secuencia de bytes ilegal

$ #85 Original Message:(Interrupted system call should be restarted)
# Llamada al sistema interrumpida debería ser reinicializada

$ #86 Original Message:(Streams pipe error)
# Streams pipe error

$ #87 Original Message:(Too many users)
# Demasiados usuarios

$ #88 Original Message:(Socket operation on non-socket)
# Operación de socket en no-socket

$ #89 Original Message:(Destination address required)
# Falta dirección de destino

$ #90 Original Message:(Message too long)
# Mensaje demasiado largo

$ #91 Original Message:(Protocol wrong type for socket)
# Protocolo de tipo erróneo para el socket

$ #92 Original Message:(Protocol not available)
# Protocolo no disponible

$ #93 Original Message:(Protocol not supported)
# Protocolo no soportado

$ #94 Original Message:(Socket type not supported)
# Tipo de socket no soportado

$ #95 Original Message:(Operation not supported on transport endpoint)
# Operación no soportada en terminal de transporte (transport endpoint)

$ #96 Original Message:(Protocol family not supported)
# Familia de protocolos no soportada

$ #97 Original Message:(Address family not supported by protocol)
# Familia de direcciones no soportada por el protocolo

$ #98 Original Message:(Address already in use)
# Dirección ya en uso

$ #99 Original Message:(Cannot assign requested address)
# No puedo asignar la dirección solicitada

$ #100 Original Message:(Network is down)
# La red está caída

$ #101 Original Message:(Network is unreachable)
# Red fuera de alcance

$ #102 Original Message:(Network dropped connection because of reset)
# La red cortó la conexión debido a un reinicio

$ #103 Original Message:(Software caused connection abort)
# Conexión abortada por programa

$ #104 Original Message:(Connection reset by peer)
# Conexión reninicializada por el otro extremo

$ #105 Original Message:(No buffer space available)
# No queda espacio para buffers

$ #106 Original Message:(Transport endpoint is already connected)
# Terminal (transport endpoint) ya conectado

$ #107 Original Message:(Transport endpoint is not connected)
# Terminal (transport endpoint) no conectado

$ #108 Original Message:(Cannot send after transport endpoint shutdown)
# No se puede enviar despues del cierre del terminal (transport endpoint shutdown)

$ #109 Original Message:(Too many references: cannot splice)
# Demasiadas referencias: cannot splice

$ #110 Original Message:(Connection timed out)
# Conexión temporizada

$ #111 Original Message:(Connection refused)
# Conexión rechazada

$ #112 Original Message:(Host is down)
# Host caído

$ #113 Original Message:(No route to host)
# No hay ruta al host

$ #114 Original Message:(Operation already in progress)
# Operación en ya progreso

$ #115 Original Message:(Operation now in progress)
# Operatión en progreso

$ #116 Original Message:(Stale NFS file handle)
# Stale NFS file handle

$ #117 Original Message:(Structure needs cleaning)
# La estructura necesita ser limpiada

$ #118 Original Message:(Not a XENIX named type file)
# No es un fichero XENIX (XENIX named type file)

$ #119 Original Message:(No XENIX semaphores available)
# No hay semáforos XENIX disponibles

$ #120 Original Message:(Is a named type file)
# Is a named type file

$ #121 Original Message:(Remote I/O error)
# Error remoto de entrada/salida
