$set 1 #ErrorList

$ #0 Original Message:(Unknown error)
# Unbekannter Fehler

$ #1 Original Message:(Operation not permitted)
# Keine Berechtigung dazu

$ #2 Original Message:(No such file or directory)
# Datei oder Verzeichnis nicht gefunden

$ #3 Original Message:(No such process)
# Prozeß nicht gefunden

$ #4 Original Message:(Interrupted system call)
# Unterbrechung während Betriebssystem-Aufruf

$ #5 Original Message:(I/O error)
# Fehler bei Schreib-/Lesezugriff

$ #6 Original Message:(No such device or address)
# Gerät oder Adresse nicht gefunden

$ #7 Original Message:(Arg list too long)
# Zu lange Argumentliste

$ #8 Original Message:(Exec format error)
# Kein ausführbares Programm

$ #9 Original Message:(Bad file number)
# unbrauchbare Datei-Zugriffs-Nummer

$ #10 Original Message:(No child processes)
# Kein Kind-Prozeß

$ #11 Original Message:(Try again)
# Erneut versuchen...

$ #12 Original Message:(Out of memory)
# Speicher-Adreßbereich oder Swapspace reicht nicht

$ #13 Original Message:(Permission denied)
# Keine Berechtigung

$ #14 Original Message:(Bad address)
# Ungültige Adresse

$ #15 Original Message:(Block device required)
# Nur block-strukturierte Geräte erlaubt

$ #16 Original Message:(Device or resource busy)
# Gerät oder Resource ist beschäftigt

$ #17 Original Message:(File exists)
# Datei existiert

$ #18 Original Message:(Cross-device link)
# Geräte-überschreitende Links sind nicht erlaubt

$ #19 Original Message:(No such device)
# Gerät nicht gefunden

$ #20 Original Message:(Not a directory)
# Das ist kein Verzeichnis

$ #21 Original Message:(Is a directory)
# Das ist ein Verzeichnis

$ #22 Original Message:(Invalid argument)
# Ungültiger Parameter

$ #23 Original Message:(File table overflow)
# Tabelle der offenen Dateien ist voll

$ #24 Original Message:(Too many open files)
# Zu viele offene Dateien

$ #25 Original Message:(Not a typewriter)
# Das Gerät ist kein Fernschreiber (tty)

$ #26 Original Message:(Text file busy)
# Programm wird gerade geändert oder ausgeführt

$ #27 Original Message:(File too large)
# Zu große Datei

$ #28 Original Message:(No space left on device)
# Kein weiterer Speicherplatz verfügbar auf dem Gerät

$ #29 Original Message:(Illegal seek)
# Ungültiger Seek

$ #30 Original Message:(Read-only file system)
# Dieses Dateisystem erlaubt keinen Schreibzugriff

$ #31 Original Message:(Too many links)
# Zu viele Links

$ #32 Original Message:(Broken pipe)
# Datenübergabe unterbrochen (Broken pipe)

$ #33 Original Message:(Math argument out of domain of func)
# Argument zu mathematischer Funktion außerhalb des Definitionsbereichs

$ #34 Original Message:(Math result not representable)
# Ergebnis mathematischer Funktion ist nicht darstellbar

$ #35 Original Message:(Resource deadlock would occur)
# Das würde zu einer Verklemmung führen

$ #36 Original Message:(File name too long)
# Dateiname zu lang

$ #37 Original Message:(No record locks available)
# Keine Satz-Sperren verfügbar

$ #38 Original Message:(Function not implemented)
# Funktion ist nicht implementiert

$ #39 Original Message:(Directory not empty)
# Verzeichnis ist nicht leer

$ #40 Original Message:(Too many symbolic links encountered)
# Zu viele symbolische Links gefunden

$ #41 Original Message:(Operation would block)
# Darauf müßte gewartet werden

$ #42 Original Message:(No message of desired type)
# Nachrichten dieses Typs gibt es hier nicht

$ #43 Original Message:(Identifier removed)
# Name (einer Semaphore) wurde gelöscht

$ #44 Original Message:(Channel number out of range)
# Kanal-Nummer außerhalb des erlaubten Wertebereiches

$ #45 Original Message:(Level 2 not synchronized)
# Stufe 2 nicht synchronisiert

$ #46 Original Message:(Level 3 halted)
# Stufe 3 angehalten

$ #47 Original Message:(Level 3 reset)
# Stufe 3 zurückgesetzt

$ #48 Original Message:(Link number out of range)
# Link-Nummer außerhalb des erlaubten Wertebereiches

$ #49 Original Message:(Protocol driver not attached)
# Protokoll Treiber nicht angeschlossen

$ #50 Original Message:(No CSI structure available)
# Keine CSI Struktur verfügbar

$ #51 Original Message:(Level 2 halted)
# Stufe 2 angehalten

$ #52 Original Message:(Invalid exchange)
# Ungültiger Austausch

$ #53 Original Message:(Invalid request descriptor)
# Ungültige Anforderungs-Beschreibung

$ #54 Original Message:(Exchange full)
# Voller Austausch

$ #55 Original Message:(No anode)
# Kein anode

$ #56 Original Message:(Invalid request code)
# Ungültiger Anforderungs-Code

$ #57 Original Message:(Invalid slot)
# Ungültiger Slot

$ #58 Original Message:(File locking deadlock error)
# Fehler: Datei-Sperre liefert eine Verklemmung

$ #59 Original Message:(Bad font file format)
# Ungültiges Datei-Format für Zeichensätze

$ #60 Original Message:(Device not a stream)
# Gerät ist kein Stream

$ #61 Original Message:(No data available)
# Keine Daten verfügbar

$ #62 Original Message:(Timer expired)
# Timer abgelaufen

$ #63 Original Message:(Out of streams resources)
# Keine Streams mehr verfügbar

$ #64 Original Message:(Machine is not on the network)
# Maschine ist nicht an das Netzwerk angeschlossen

$ #65 Original Message:(Package not installed)
# Paket ist nicht installiert

$ #66 Original Message:(Object is remote)
# Objekt ist auf einem entfernten Rechner

$ #67 Original Message:(Link has been severed)
# Link zerschnitten

$ #68 Original Message:(Advertise error)
# Advertise Fehler

$ #69 Original Message:(Srmount error)
# Srmount Fehler

$ #70 Original Message:(Communication error on send)
# Kommunikations-Fehler beim Senden

$ #71 Original Message:(Protocol error)
# Protokoll Fehler

$ #72 Original Message:(Multihop attempted)
# Multihop versucht

$ #73 Original Message:(RFS specific error)
# RFS spezifischer Fehler

$ #74 Original Message:(Not a data message)
# Keine Daten-Nachricht

$ #75 Original Message:(Value too large for defined data type)
# Wert außerhalb des Datenbereiches für den definierten Datentyp

$ #76 Original Message:(Name not unique on network)
# Name ist nicht eindeutig im Netzwerk

$ #77 Original Message:(File descriptor in bad state)
# Datei-Zugriffsnummer ist in einem ungültigen Zustand

$ #78 Original Message:(Remote address changed)
# Entfernte Adresse hat sich geändert

$ #79 Original Message:(Can not access a needed shared library)
# Benötigte Shared-Library nicht im Zugriff

$ #80 Original Message:(Accessing a corrupted shared library)
# Zugriff auf eine defekte Shared-Library

$ #81 Original Message:(.lib section in a.out corrupted)
# .lib Sektion in a.out defekt

$ #82 Original Message:(Attempting to link in too many shared libraries)
# Link-Versuch für zu viele Shared-Libraries

$ #83 Original Message:(Cannot exec a shared library directly)
# Eine Shared-Library kann nicht direkt ausgeführt werden

$ #84 Original Message:(Illegal byte sequence)
# Ungültige Byte-Folge

$ #85 Original Message:(Interrupted system call should be restarted)
# Unterbrochener Betriebssystem-Aufruf sollte neu gestartet werden

$ #86 Original Message:(Streams pipe error)
# Streams pipe Fehler

$ #87 Original Message:(Too many users)
# Zu viele Benutzer

$ #88 Original Message:(Socket operation on non-socket)
# Socket Operation an einem nicht-Socket

$ #89 Original Message:(Destination address required)
# Zieladresse muß angegeben werden

$ #90 Original Message:(Message too long)
# Nachricht zu lang

$ #91 Original Message:(Protocol wrong type for socket)
# Protokoll-Typ fehlerhaft für einen Socket

$ #92 Original Message:(Protocol not available)
# Protokoll nicht verfügbar

$ #93 Original Message:(Protocol not supported)
# Protokoll nicht unterstützt

$ #94 Original Message:(Socket type not supported)
# Socket Typ nicht unterstützt

$ #95 Original Message:(Operation not supported on transport endpoint)
# Operation nicht unterstützt am Transport-Endpunkt

$ #96 Original Message:(Protocol family not supported)
# Protokoll-Familie nicht unterstützt

$ #97 Original Message:(Address family not supported by protocol)
# Adress-Familie nicht unterstützt vom Protokoll

$ #98 Original Message:(Address already in use)
# Adresse wird bereits benutzt

$ #99 Original Message:(Cannot assign requested address)
# Die gewünschte Adresse kann nicht benutzt werden

$ #100 Original Message:(Network is down)
# Netzwerk ist nicht operabel

$ #101 Original Message:(Network is unreachable)
# Netzwerk ist nicht erreichbar

$ #102 Original Message:(Network dropped connection because of reset)
# Netwerk-Verbindung wegen Reset beendet

$ #103 Original Message:(Software caused connection abort)
# Verbindung durch das Programm beendet

$ #104 Original Message:(Connection reset by peer)
# Verbindung durch den Kommunikationspartner zurückgesetzt

$ #105 Original Message:(No buffer space available)
# Kein Puffer-Speicher verfügbar

$ #106 Original Message:(Transport endpoint is already connected)
# Transport-Endpunkt ist bereits verbunden

$ #107 Original Message:(Transport endpoint is not connected)
# Transport-Endpunkt ist nicht verbunden

$ #108 Original Message:(Cannot send after transport endpoint shutdown)
# Senden nach Beenden des Transport-Endpunktes nicht möglich

$ #109 Original Message:(Too many references: cannot splice)
# Zu viele Referenzen: splice ist nicht möglich

$ #110 Original Message:(Connection timed out)
# Verbindung wegen Zeitüberschreitung abgebrochen

$ #111 Original Message:(Connection refused)
# Verbindungsaufbau abgelehnt

$ #112 Original Message:(Host is down)
# Netzwerkrechner ist nicht verfügbar

$ #113 Original Message:(No route to host)
# Keine Route zum Netzwerkrechner bekannt

$ #114 Original Message:(Operation already in progress)
# Operation ist bereits in Bearbeitung

$ #115 Original Message:(Operation now in progress)
# Operation ist jetzt in Bearbeitung

$ #116 Original Message:(Stale NFS file handle)
# Stale NFS Datei-Zugriffs-Nummer

$ #117 Original Message:(Structure needs cleaning)
# Struktur muß gereinigt werden (needs cleaning)

$ #118 Original Message:(Not a XENIX named type file)
# Keine XENIX named-type Datei

$ #119 Original Message:(No XENIX semaphores available)
# Keine XENIX Semaphoren verfügbar

$ #120 Original Message:(Is a named type file)
# Ist eine named-type Datei

$ #121 Original Message:(Remote I/O error)
# Ein-/Ausgabe-Fehler auf einem entfernten Rechner
