/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Tue May 23 15:25:54 1995
 */
extern double acos ( double x );
extern double acosh ( double x );
extern double asin ( double x );
extern double asinh ( double x );
extern double atan ( double x );
extern double atan2 ( double y, double x );
extern double atanh ( double x );
extern double cbrt ( double x );
extern double ceil ( double x );
extern double chbevl ( double x, void * array, int n );
extern double cos ( double x );
extern double cosh ( double x );
extern double cosm1 ( double x );
extern double cot ( double x );
extern int drand ( double *a );
extern double exp ( double x );
extern double exp10 ( double x );
extern double expm1 ( double x );
extern double fabs ( double x );
extern double floor ( double x );
extern double frexp ( double x, int *pw2 );
extern int isfinite ( double x );
extern int isnan ( double x );
extern double ldexp ( double x, int pw2 );
extern double log ( double x );
extern double log10 ( double x );
extern double log1p ( double x );
extern int mtherr ( char *name, int code );
extern double p1evl ( double x, void * coef, int N );
extern double polevl ( double x, void * coef, int N );
extern double pow ( double x, double y );
extern double powi ( double x, int nn );
extern double round ( double x );
extern int signbit ( double x );
extern double sin ( double x );
extern int sincos ( double x, double *s, double *c, int flg );
extern double sinh ( double x );
extern double sqrt ( double x );
extern double tan ( double x );
extern double tanh ( double x );
extern double cabs ( cmplx *z );
extern double hypot ( double x, double y );
extern void cneg ( cmplx *z );
extern void cmov ( void *z, void *w );
extern void cadd ( cmplx *a, cmplx *b, cmplx *c );
extern void csub ( cmplx *a, cmplx *b, cmplx *c );
extern void cmul ( cmplx *a, cmplx *b, cmplx *c );
extern void cdiv ( cmplx *a, cmplx *b, cmplx *c );
extern void csqrt ( cmplx *z, cmplx *w );
extern void clog ( cmplx *z, cmplx *w );
extern void cexp ( cmplx *z, cmplx *w );
extern void csin ( cmplx *z, cmplx *w );
extern void ccos ( cmplx *z, cmplx *w );
extern void ctan ( cmplx *z, cmplx *w );
extern void ccot ( cmplx *z, cmplx *w );
extern void casin ( cmplx *z, cmplx *w );
extern void cacos ( cmplx *z, cmplx *w );
extern void catan ( cmplx *z, cmplx *w );
