$set 7 #AuthList

$ #1 Original Message:(Authentication OK)
# Authentication réussie

$ #2 Original Message:(Invalid client credential)
# L'identité du client est inconnue

$ #3 Original Message:(Server rejected credential)
# Le serveur a rejeté l'identité

$ #4 Original Message:(Invalid client verifier)
# Le vérificateur du client est invalide

$ #5 Original Message:(Server rejected verifier)
# Le serveur a rejeté le vérificateur

$ #6 Original Message:(Client credential too weak)
# L'identité du client est trop faible

$ #7 Original Message:(Invalid server verifier)
# Le vérificateur du serveur est invalide

$ #8 Original Message:(Failed (unspecified error))
# Echec (erreur non spécifiée)
