$set 11 #NetMisc

$ #AllPortsInUse Original Message:(All ports in use)
# Alle Ports sind belegt

$ #ConnectTo Original Message:(connect to address)
# Verbunden zur Adresse

$ #Trying Original Message:(Trying %s...\n)
# Versuche %s...\n

$ #SettingUpStderr Original Message:(write: setting up stderr)
# Write: Standard-Fehler-Ausgabe eingerichtet

$ #Accept Original Message:(accept)
# angenommen

$ #Getsockname Original Message:(getsockname)
# GetSockName

$ #ProtocolFailure Original Message:(socket: protocol failure in circuit setup.)
# Socket: Protokoll-Fehler im Kreislauf-Setup.

$ #RcmdSocket Original Message:(rcmd: socket)
# rcmd: Socket

$ #RexecSocket Original Message:(rexec: socket)
# rexec: Socket

$ #NetrcWrongPasswordMode Original Message:(Error - .netrc file not correct mode.\nRemove password or correct mode.)
# Fehler - .netrc Datei hat nicht den richtigen Datei-Mode.\nPasswort löschen oder Mode korrigieren.

$ #UnknownNetrcOption Original Message:(Unknown .netrc option)
# Unbekannte .netrc Option

$ #ResolvIncorrectFormat Original Message:(resolv+: %s: \"%s\" command incorrectly formatted.\n)
# resolv+: %s: "%s" Kommando fehlerhaft formatiert.\n

$ #ResolvInvalid Original Message:(resolv+: \"%s\" is an invalid keyword\n)
# resolv+: "%s" ist ein ungültiges Schlüselwort\n

$ #ResolvValid Original Message:(resolv+: valid keywords are: %s, %s and %s\n)
# resolv+: Gültige Schlüsselworte sind: %s, %s und %s\n

$ #Unrecognized Original Message:(resolv+: search order not specified or unrecognized keyword, host resolution will fail.\n)
# resolv+: Suchreihenfolge nicht bestimmt oder unbekanntes Schlüsselwort,\nHost-Namenauflösung schlägt fehl.

$ #PossibleSpoof Original Message:(gethostbyaddr: %s != %u.%u.%u.%u, possible spoof attempt)
# gethostbyaddr: %s != %u.%u.%u.%u, möglicherweise spoof attempt

$ #SelectSetup Original Message:(rcmd: select (setting up stderr): )
# rcmd: Select (Einrichten der Standard-Fehler-Ausgabe): 

$ #RcmdProtocolError Original Message:(select: protocol failure in circuit setup)
# select: Protokoll Fehler in der Kreislauf Einstellung

$ #RhostLstat Original Message:(.rhosts lstat failed)
# .rhosts lstat fehlgeschlagen

$ #RhostNotRegular Original Message:(.rhosts not regular file)
# .rhosts ist keine reguläre Datei

$ #BadRhostsOwner Original Message:(bad .rhosts owner)
# ungültiger .rhosts Eigentümer

$ #RhostFstatFailed Original Message:(.rhosts fstat failed)
# .rhosts fstat fehlgeschlagen

$ #RhostWritable Original Message:(.rhosts writeable by other than owner)
# .rhosts ist nicht nur vom Eigner schreibbar

$ #Name Original Message:(Name)
# Name

$ #Password Original Message:(Password)
# Passwort

$ #AskedForGot Original Message:(gethostby*.getanswer: asked for \"%s\", got \"%s\")
# gethostby*.getanswer: nach "%s" gefragt, "%s" erhalten

$ #AskedForGotCNAME Original Message:(gethostby*.getanswer: asked for \"%s\", got CNAME for \"%s\")
# gethostby*.getanswer: nach "%s" gefragt, CNAME "%s" erhalten

$ #AskedForType Original Message:(gethostby*.getanswer: asked for type %d(%s), got %d(%s))
# gethostby*.getanswer: nach Typ für %d(%s) gefragt, %d(%s) erhalten

